// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_NET_H
#define OB_NET_H

#ifdef HAVE_WINSOCK_H

#   include <winsock.h>

namespace OB
{

inline int GetError()
{
    return WSAGetLastError();
}

inline void SetError(int err)
{
    WSASetLastError(err);
}

inline int CreateSocket()
{
    return socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
}

inline int CloseSocket(int fd)
{
    return closesocket(fd);
}

inline bool ConnectionDown()
{
    return GetError() == WSAECONNRESET;
}

inline bool WouldBlock()
{
    return GetError() == WSAEWOULDBLOCK;
}

inline bool InProgress()
{
    return GetError() == WSAEWOULDBLOCK;
}

inline bool AcceptRetry()
{
    return GetError() == WSAEINTR;
}

inline bool IsInterrupted()
{
    return GetError() == WSAEINTR;
}

inline void SetBlock(int fd)
{
    unsigned long arg = 0;
    ioctlsocket(fd, FIONBIO, &arg);
}

inline void SetNoBlock(int fd)
{
    unsigned long arg = 1;
    ioctlsocket(fd, FIONBIO, &arg);
}

} // End of namespace OB

#else

#   include <stdlib.h>
#   include <stdio.h>
#   include <unistd.h>
#   include <errno.h>
#   include <signal.h>
#   include <fcntl.h>
#   include <sys/types.h>
#   include <sys/param.h>
#   include <sys/time.h>
#   include <sys/socket.h>
#   ifdef HAVE_BSTRING_H
#       include <bstring.h>
#   endif
#   ifdef HAVE_STRINGS_H
#       include <strings.h>
#   endif
#   ifdef HAVE_SYS_SELECT_H
#	include <sys/select.h>
#   endif
#   include <netinet/in.h>
#   include <netinet/tcp.h>
#   include <netdb.h>
#   include <arpa/inet.h>

namespace OB
{

inline int GetError()
{
    return errno;
}

inline void SetError(int err)
{
    errno = err;
}

inline int CreateSocket()
{
    return socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
}

inline int CloseSocket(int fd)
{
    return close(fd);
}

inline bool ConnectionDown()
{
    return GetError() == ECONNRESET || GetError() == EPIPE;
}

inline bool WouldBlock()
{
    return GetError() == EAGAIN || GetError() == EWOULDBLOCK;
}

inline bool InProgress()
{
    return GetError() == EINPROGRESS;
}

inline bool AcceptRetry()
{
    switch(GetError())
    {
    case EINTR:
	
#ifdef EPROTO
	//
	// This error can occur on Solaris.  accept() returns EPROTO
	// is a connection was closed remotely before it was accepted.
	//
    case EPROTO:
#endif
        
#ifdef ECONNABORTED
	//
	// In Solaris 2.6 and later, you may get ECONNABORTED (instead
	// of EPROTO) under certain race conditions and that is the
	// error code in the Posix standard too.  So you need to do
	// "ignore-and-retry" for EINTR, ECONNABORTED and EPROTO if you
	// need portability across releases of Solaris. If you need to
	// be portable across several variants of Linux/BSD/Unix etc,
	// you may need to do some #ifdef's since even the constant
	// EPROTO/ECONNABORTED may not be there on some platforms. A
	// good example code to look at is the code around the
	// "accept()" call in the Apache Web server source code
	// available from the site http://www.apache.org
	//
    case ECONNABORTED:
#endif
        
#ifdef ECONNRESET
    case ECONNRESET:
#endif
        
#ifdef ETIMEDOUT
    case ETIMEDOUT:
#endif
        return true;
    }

    return false;
}

inline bool IsInterrupted()
{
    switch(GetError())
    {
    case EINTR:

	//
	// Not positive on this, but it appears from some newsgroup
	// postings that some OS's #define EAGAIN EWOULDBLOCK
	//
#if EAGAIN != EWOULDBLOCK
    case EAGAIN:
#endif

#ifdef EPROTO
    case EPROTO:
#endif
        
	return true;
    }

    return false;
}

inline void SetBlock(int fd)
{
    int flags = fcntl(fd, F_GETFL);
    flags &= ~O_NONBLOCK;
    fcntl(fd, F_SETFL, flags);
}

inline void SetNoBlock(int fd)
{
    int flags = fcntl(fd, F_GETFL);
    flags |= O_NONBLOCK;
    fcntl(fd, F_SETFL, flags);
}

} // End of namespace OB

#endif

#ifdef HAVE_NO_GETHOSTNAME_PROTOTYPE
extern "C"
{
    int gethostname(char*,  int);
}
#endif

#ifdef HAVE_NO_H_ERRNO_DECLARATION
extern "C"
{
    extern int h_errno;
}
#endif

#ifndef MAXHOSTNAMELEN
#   define MAXHOSTNAMELEN 1024
#endif

#ifndef IPPORT_MAXPORT
#   define IPPORT_MAXPORT 65535
#endif

#ifndef IPPORT_USERRESERVED
#   define IPPORT_USERRESERVED 5000
#endif

#ifndef INADDR_ANY
#   define INADDR_ANY ((unsigned) 0x00000000)
#endif

#ifndef INADDR_NONE
#   define INADDR_NONE ((unsigned) 0xffffffff)
#endif

#ifndef INADDR_LOOPBACK
#   define INADDR_LOOPBACK 0x7f000001 /* Internet address 127.0.0.1 */
#endif

#define OB_GETHOSTBYNAME_RETRY 3

namespace OB
{

//
// Get an error message for a given errno (hides differences between
// Unix and NT)
//
const char* LastError(); // For "regular" errors
const char* LastHError(); // For gethostbyname()-related errors

//
// Get internet address
//
void GetInAddr(const char*, struct sockaddr_in&);

} // End of namespac OB

#endif
